<?php

namespace Drupal\slimpay_api;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use HapiClient\Hal\CustomRel;
use HapiClient\Hal\ResourceInterface;
use HapiClient\Http\Auth\Oauth2BasicAuthentication;
use HapiClient\Http\Follow;
use HapiClient\Http\HapiClient;
use HapiClient\Http\JsonBody;
use Psr\Log\LoggerInterface;

/**
 * Provides a client for Slim Pay Web API.
 */
class Client {

  /**
   * The URL for the HAL profile containing the resources and their descriptors.
   */
  const PROFILE_URL = 'https://api.slimpay.net/alps/v1';

  /**
   * The relations namespace.
   */
  const RELATIONS_NAMESPACE = 'https://api.slimpay.net/alps#';

  /**
   * The hypermedia API client.
   *
   * @var \HapiClient\Http\HapiClient
   */
  protected $hapiClient;

  /**
   * A config object for the slim pay configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a Client object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger instance.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    LoggerInterface $logger,
    FileSystemInterface $file_system
  ) {
    $this->config = $config_factory->get('slimpay_api.settings');
    $this->logger = $logger;
    $this->fileSystem = $file_system;
    $this->hapiClient = new HapiClient(
      $this->config->get('api_url'),
      '/',
      self::PROFILE_URL,
      new Oauth2BasicAuthentication(
        '/oauth/token',
        $this->config->get('app_id'),
        $this->config->get('app_secret')
      )
    );
  }

  /**
   * Creates an order.
   *
   * @param string $reference
   *   Reference.
   * @param string $subscriber_id
   *   Subscriber ID.
   * @param string $street1
   *   Street address line 1.
   * @param string $street2
   *   Street address line 2.
   * @param string $city
   *   City.
   * @param string $postal_code
   *   Postal code.
   * @param string $country
   *   Country.
   * @param string $honorific_prefix
   *   Honorific prefix.
   * @param string $family_name
   *   Family name.
   * @param string $given_name
   *   Given name.
   * @param string $email
   *   Email.
   * @param string $telephone
   *   Phone number.
   * @param string $iban
   *   International Bank Account Number.
   * @param string $customer_number
   *   Customer's ID number.
   * @param string $success_url
   *   The URL loaded after successful user approval.
   * @param string $cancel_url
   *   The URL loaded after user cancels approval.
   * @param string $failure_url
   *   The URL loaded after user approval fails.
   *
   * @throws \HapiClient\Exception\HttpException
   *   Thrown when Slimpay rises an error.
   * @throws \Exception
   *
   * @return \HapiClient\Hal\ResourceInterface|null
   *   The order resource on success. Otherwise, NULL.
   */
  public function createOrders(
    $reference,
    $subscriber_id,
    $street1,
    $street2,
    $city,
    $postal_code,
    $country,
    $honorific_prefix,
    $family_name,
    $given_name,
    $email,
    $telephone,
    $iban,
    $customer_number = '',
    $success_url = '',
    $cancel_url = '',
    $failure_url = ''
  ) {
    $rel = new CustomRel(self::RELATIONS_NAMESPACE . 'create-orders');
    $signatory = [
      'billingAddress' => [
        'street1' => $street1,
        'street2' => $street2,
        'city' => $city,
        'postalCode' => $postal_code,
        'country' => $country,
      ],
      'honorificPrefix' => $honorific_prefix,
      'familyName' => $family_name,
      'givenName' => $given_name,
      'email' => $email,
      'telephone' => $telephone,
    ];
    $body = [
      'reference' => $reference,
      'paymentScheme' => $this->config->get('payment_scheme'),
      'locale' => 'fr',
      'started' => 1,
      'creditor' => ['reference' => $this->config->get('creditor')],
      'subscriber' => ['reference' => $subscriber_id],
      'items' => [
        [
          'type' => 'signMandate',
          'action' => 'sign',
          'autoGenReference' => 1,
          'mandate' => [
            'signatory' => $signatory + [
              'bankAccount' => ['iban' => $iban],
            ],
            'standard' => 'SEPA',
            'paymentScheme' => $this->config->get('payment_scheme'),
          ],
          'signatureApproval' => [
            'method' => ['type' => 'otp'],
          ],
        ],
        [
          'type' => 'templateBasedDocument',
          'action' => 'sign',
          'reference' => 'contrat_abonnement_nespresso',
          'documentTemplate' => [
            'reference' => $this->config->get('template'),
          ],
          'extraParams' => [
            'customer_number' => $customer_number,
          ],
          'signatory' => $signatory,
        ],
      ],
    ];
    if (!empty($success_url) && !empty($failure_url) && !empty($cancel_url)) {
      $body += [
        'successUrl' => $success_url,
        'cancelUrl' => $cancel_url,
        'failureUrl' => $failure_url,
      ];
    }
    $follow = new Follow($rel, 'POST', NULL, new JsonBody($body));
    return $this->hapiClient->sendFollow($follow);
  }

  /**
   * Get extended user approval with an iframe displayed on the merchant site.
   *
   * @param \HapiClient\Hal\ResourceInterface $order
   *   Order for approve.
   *
   * @throws \HapiClient\Exception\HttpException
   *   Thrown when Slimpay rises an error.
   * @throws \Exception
   *
   * @return \HapiClient\Hal\ResourceInterface|null
   *   The extended approval resource on success. Otherwise, NULL.
   */
  public function extendedUserApproval(ResourceInterface $order) {
    $rel = new CustomRel(self::RELATIONS_NAMESPACE . 'extended-user-approval');
    $follow = new Follow($rel, 'GET', [
      'mode' => $this->config->get('iframe_mode'),
    ]);
    return $this->hapiClient->sendFollow($follow, $order);
  }

  /**
   * Gets an order by reference code.
   *
   * @param string $reference
   *   Order's reference code.
   *
   * @return \HapiClient\Hal\ResourceInterface|null
   *   The order resource on success. Otherwise, NULL.
   */
  public function getOrder(string $reference) {
    $rel = new CustomRel(self::RELATIONS_NAMESPACE . 'get-orders');
    $follow = new Follow($rel, 'GET', [
      'creditorReference' => $this->config->get('creditor'),
      'reference' => $reference,
    ]);

    try {
      return $this->hapiClient->sendFollow($follow);
    }
    catch (\Exception $e) {
      $this->logger->critical(
        "Error while getting order: %error",
        ['%error' => $e->getMessage()]
      );
    }

    return NULL;
  }

  /**
   * Gets the document linked to a given order.
   *
   * @param \HapiClient\Hal\ResourceInterface $order
   *   The order.
   *
   * @return \HapiClient\Hal\ResourceInterface|null
   *   The document resource on success. Otherwise, NULL.
   */
  public function getDocument(ResourceInterface $order) {
    $rel = new CustomRel(self::RELATIONS_NAMESPACE . 'get-document');
    $follow = new Follow($rel);

    try {
      return $this->hapiClient->sendFollow($follow, $order);
    }
    catch (\Exception $e) {
      $this->logger->critical(
        "Error while getting document: %error",
        ['%error' => $e->getMessage()]
      );
    }

    return NULL;
  }

  /**
   * Gets the mandate linked to a given order.
   *
   * @param \HapiClient\Hal\ResourceInterface $order
   *   Order for approve.
   *
   * @return \HapiClient\Hal\ResourceInterface|null
   *   The mandate resource on success. Otherwise, NULL.
   */
  public function getMandate(ResourceInterface $order) {
    $rel = new CustomRel(self::RELATIONS_NAMESPACE . 'get-mandate');
    $follow = new Follow($rel);

    try {
      return $this->hapiClient->sendFollow($follow, $order);
    }
    catch (\Exception $e) {
      $this->logger->critical(
        "Error while getting mandate: %error",
        ['%error' => $e->getMessage()]
      );
    }

    return NULL;
  }

  /**
   * Creates a file entity from a binary document resource.
   *
   * @param \HapiClient\Hal\ResourceInterface $resource
   *   A binary document resource.
   * @param string $directory
   *   URI of directory holding the imported document.
   *
   * @return \Drupal\file\FileInterface|false
   *   A file entity, or FALSE on error.
   */
  public function getBinaryContent(
    ResourceInterface $resource,
    string $directory = 'temporary://'
  ) {
    $rel = new CustomRel(self::RELATIONS_NAMESPACE . 'get-binary-content');
    $follow = new Follow($rel);

    try {
      $binary = $this->hapiClient->sendFollow($follow, $resource);
    }
    catch (\Exception $e) {
      $this->logger->critical(
        "Error while getting binary content: %error",
        ['%error' => $e->getMessage()]
      );
    }

    if (!empty($binary)) {
      $writable = $this->fileSystem->prepareDirectory(
        $directory,
        FileSystemInterface::CREATE_DIRECTORY
      );
      if ($writable) {
        return file_save_data(
          base64_decode($binary->getState()['content']),
          "{$directory}/{$resource->getState()['id']}.pdf",
          FILE_EXISTS_REPLACE
        );
      }
      else {
        $this->logger->error(
          "Directory %directory is not writable.",
          ['%directory' => $directory]
        );
      }
    }
    return FALSE;
  }

}
