<?php

namespace Drupal\slimpay_api\Commands;

use Drupal\slimpay_api\Client;
use Drush\Commands\DrushCommands;

/**
 * Drush commands concerning centers.
 */
class HypermediaApiCommands extends DrushCommands {

  /**
   * The Slimpay API client.
   *
   * @var \Drupal\slimpay_api\Client
   */
  protected $client;

  /**
   * HypermediaApi constructor.
   *
   * @param \Drupal\slimpay_api\Client $client
   *   The Slimpay API client.
   */
  public function __construct(Client $client) {
    parent::__construct();
    $this->client = $client;
  }

  /**
   * Creates an order.
   *
   * @command slimpay-api:create-orders
   *
   * @usage drush slimpay-api:create-orders 'REFERENCE' 'SUBSCRIBER' '27 rue des fleurs' 'Bat 2' 'Paris' '75008' 'FR' 'Mr' 'Ferlay' 'David' 'dferlay@skilld.fr' '+33670110375' 'GB33BUKB20201555555555' 'CUSTOMER NUMBER' 'https://www.success.com' 'https://www.cancel.com' 'https://www.failure.com'
   *   Creates an order with the given data.
   */
  public function createOrders(
    $reference,
    $subscriber_id,
    $street1,
    $street2,
    $city,
    $postal_code,
    $country,
    $honorific_prefix,
    $family_name,
    $given_name,
    $email,
    $telephone,
    $iban,
    $customer_number,
    $success_url,
    $cancel_url,
    $failure_url
  ) {
    $order = $this->client->createOrders(
      $reference,
      $subscriber_id,
      $street1,
      $street2,
      $city,
      $postal_code,
      $country,
      $honorific_prefix,
      $family_name,
      $given_name,
      $email,
      $telephone,
      $iban,
      $customer_number,
      $success_url,
      $cancel_url,
      $failure_url
    );
    $this->logger()->success(
      'Response: {response}',
      ['response' => implode('|', $order->getState())]
    );
  }

  /**
   * Get an extended user approval for a given order.
   *
   * @command slimpay-api:extended-user-approval
   *
   * @usage drush slimpay-api:extended-user-approval REFERENCE
   *   Gets extended user approval for order with reference code REFERENCE.
   */
  public function extendedUserApproval($reference) {
    $order = $this->client->getOrder($reference);
    if (empty($order)) {
      return $this->logger()->error('Order not found.');
    }
    $extended_approval = $this->client->extendedUserApproval($order);
    if (empty($extended_approval)) {
      return $this->logger()->error('Failed to get extended approval.');
    }
    $this->logger()->success(
      'Response: {html}',
      ['html' => base64_decode($extended_approval->getState()['content'])]
    );
  }

  /**
   * Gets an order by reference code.
   *
   * @command slimpay-api:get-order
   *
   * @usage drush slimpay-api:get-order REFERENCE1
   *   Gets the order with reference code REFERENCE1.
   */
  public function getOrder(string $reference) {
    $order = $this->client->getOrder($reference);
    if (empty($order)) {
      return FALSE;
    }
    $this->logger()->success(
      'Response: {response}',
      ['response' => implode('|', $order->getState())]
    );
  }

  /**
   * Gets the document linked to a given order.
   *
   * @command slimpay-api:get-document
   *
   * @usage drush slimpay-api:get-document 066984b8-6cf0-11e9-9bf5-000000000000
   *   Gets the document for order with reference code
   *   066984b8-6cf0-11e9-9bf5-000000000000.
   */
  public function getDocument(string $order_reference) {
    $order = $this->client->getOrder($order_reference);
    if (empty($order)) {
      return FALSE;
    }
    $document = $this->client->getDocument($order);
    if (empty($document)) {
      return FALSE;
    }
    $this->logger()->success(
      'Response: {response}',
      ['response' => implode('|', $document->getState())]
    );
  }

  /**
   * Gets a mandate by order reference code.
   *
   * @command slimpay-api:get-mandate
   *
   * @usage drush slimpay-api:get-mandate bbd77e87-6cf5-11e9-9bf5-000000000000
   *   Gets the mandate linked to order with reference code
   *   bbd77e87-6cf5-11e9-9bf5-000000000000.
   */
  public function getMandate(string $order_reference) {
    $order = $this->client->getOrder($order_reference);
    if (empty($order)) {
      return FALSE;
    }
    $mandate = $this->client->getMandate($order);
    if (empty($mandate)) {
      return FALSE;
    }
    $this->logger()->success(
      'Response: {response}',
      ['response' => implode('|', $mandate->getState())]
    );
  }

  /**
   * Imports PDF file for a mandate by reference code.
   *
   * @command slimpay-api:get-mandate-binary
   *
   * @usage drush slimpay-api:get-mandate-binary bbd77e87-6cf5-11e9-9bf5-000000000000
   *   Imports PDF file for the mandate linked to order with reference
   *   bbd77e87-6cf5-11e9-9bf5-000000000000.
   */
  public function getMandateBinaryContent(string $order_reference) {
    $order = $this->client->getOrder($order_reference);
    if (empty($order)) {
      return FALSE;
    }
    $mandate = $this->client->getMandate($order);
    if (empty($mandate)) {
      return FALSE;
    }
    $file = $this->client->getBinaryContent($mandate);
    if (!$file) {
      return FALSE;
    }
    $this->logger()->success(
      'Mandate file created at {file_uri}',
      ['file_uri' => $file->getFileUri()]
    );
  }

  /**
   * Imports PDF file for a document by reference code.
   *
   * @command slimpay-api:get-document-binary
   *
   * @usage drush slimpay-api:get-document-binary bbd77e87-6cf5-11e9-9bf5-000000000000
   *   Imports PDF file for the document linked to order with reference
   *   bbd77e87-6cf5-11e9-9bf5-000000000000.
   */
  public function getDocumentBinaryContent(string $order_reference) {
    $order = $this->client->getOrder($order_reference);
    if (empty($order)) {
      return FALSE;
    }
    $document = $this->client->getDocument($order);
    if (empty($document)) {
      return FALSE;
    }
    $file = $this->client->getBinaryContent($document);
    if (!$file) {
      return FALSE;
    }
    $this->logger()->success(
      'Document file created at {file_uri}',
      ['file_uri' => $file->getFileUri()]
    );
  }

}
