<?php

namespace Drupal\slimpay_api\Plugin\Commerce\PaymentGateway;

use Drupal\Component\Datetime\TimeInterface;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Path\PathValidatorInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\file\FileInterface;
use Drupal\slimpay_api\Client;
use HapiClient\Hal\Resource;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Provides Slimpay's Direct debit gateway.
 *
 * @CommercePaymentGateway(
 *   id = "slimpay_api_direct_debit",
 *   label = @Translation("SlimPay (Direct Debit)"),
 *   display_label = @Translation("Direct Debit"),
 *    forms = {
 *     "offsite-payment" = "Drupal\slimpay_api\PluginForm\DirectDebitForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "visa", "mastercard", "americanexpress",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class DirectDebit extends OffsitePaymentGatewayBase {

  /**
   * The request object.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The path validator.
   *
   * @var \Drupal\Core\Path\PathValidatorInterface
   */
  protected $pathValidator;

  /**
   * The Slimpay API client.
   *
   * @var \Drupal\slimpay_api\Client
   */
  protected $client;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a new DirectDebit object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_payment\PaymentTypeManager $payment_type_manager
   *   The payment type manager.
   * @param \Drupal\commerce_payment\PaymentMethodTypeManager $payment_method_type_manager
   *   The payment method type manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack object.
   * @param \Drupal\Core\Path\PathValidatorInterface $path_validator
   *   The path validator.
   * @param \Drupal\slimpay_api\Client $client
   *   The Slimpay API client.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    PaymentTypeManager $payment_type_manager,
    PaymentMethodTypeManager $payment_method_type_manager,
    TimeInterface $time,
    RequestStack $request_stack,
    PathValidatorInterface $path_validator,
    Client $client,
    FileSystemInterface $file_system
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $entity_type_manager,
      $payment_type_manager,
      $payment_method_type_manager,
      $time
    );
    $this->requestStack = $request_stack;
    $this->pathValidator = $path_validator;
    $this->client = $client;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('request_stack'),
      $container->get('path.validator'),
      $container->get('slimpay_api.client'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'warning' => '',
      'failure_url' => '',
      'cancel_url' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state
  ) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['warning'] = [
      '#title' => $this->t('Warning message'),
      '#type' => 'textarea',
      '#default_value' => $this->configuration['warning'],
    ];
    $scheme_http_host = $this->requestStack
      ->getCurrentRequest()
      ->getSchemeAndHttpHost();
    $form['failure_url'] = [
      '#title' => $this->t('Failure URL'),
      '#type' => 'textfield',
      '#default_value' => $this->configuration['failure_url'],
      '#field_prefix' => $scheme_http_host,
    ];
    $form['cancel_url'] = [
      '#title' => $this->t('Cancel URL'),
      '#type' => 'textfield',
      '#default_value' => $this->configuration['cancel_url'],
      '#field_prefix' => $scheme_http_host,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $values = $form_state->getValue($form['#parents']);
    if (!$this->pathValidator->isValid($values['failure_url'])) {
      $form_state->setError(
        $form['failure_url'],
        $this->t('Failure URL is invalid')
      );
    }
    if (!$this->pathValidator->isValid($values['cancel_url'])) {
      $form_state->setError(
        $form['cancel_url'],
        $this->t('Cancel URL is invalid')
      );
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $values = $form_state->getValue($form['#parents']);
    $this->configuration['warning'] = $values['warning'];
    $this->configuration['failure_url'] = $values['failure_url'];
    $this->configuration['cancel_url'] = $values['cancel_url'];
  }

  /**
   * Gets the URL for the page to show when payment fails.
   */
  public function getFailureUrl() {
    return $this->pathValidator
      ->getUrlIfValid($this->configuration['failure_url'])
      ->setAbsolute()
      ->toString();
  }

  /**
   * Gets the URL for the page to show when the payment process gets cancelled.
   */
  public function getCancelUrl() {
    return $this->pathValidator
      ->getUrlIfValid($this->configuration['cancel_url'])
      ->setAbsolute()
      ->toString();
  }

  /**
   * {@inheritdoc}
   */
  public function onNotify(Request $request) {
    $slimpay_order = Resource::fromJson(
      $request->getContent()
    );
    $slimpay_order_state = $slimpay_order->getState();

    /** @var \Drupal\commerce_payment\PaymentStorage $payment_storage */
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $payment_storage->loadByRemoteId(
      $slimpay_order_state['reference']
    );

    // Only proceed if payment is created.
    if ($payment === NULL) {
      return;
    }

    if (strpos($slimpay_order_state['state'], 'closed.aborted') === 0) {
      $payment_transition = $payment->getState()
        ->getWorkflow()
        ->getTransition('void');
      $payment->getState()->applyTransition($payment_transition);
    }
    elseif (strpos($slimpay_order_state['state'], 'closed.completed') === 0) {
      $payment_transition = $payment->getState()
        ->getWorkflow()
        ->getTransition('authorize_capture');
      $payment->getState()->applyTransition($payment_transition);
      /** @var \Drupal\commerce_order\Entity\OrderInterface $commerce_order */
      $commerce_order = $payment->getOrder();
      if ($slimpay_mandate = $this->client->getMandate($slimpay_order)) {
        $slimpay_mandate_state = $slimpay_mandate->getState();
        $commerce_order->mandate_unique_reference->value = $slimpay_mandate_state['rum'];
        $signature_date = DrupalDateTime::createFromFormat(
          "Y-m-d\TH:i:s.uO",
          $slimpay_mandate_state['dateSigned'],
          NULL,
          ['validate_format' => FALSE]
        );
        $commerce_order->mandate_signature_date->value = $signature_date
          ->setTimezone(
            new \DateTimeZone(DateTimeItemInterface::STORAGE_TIMEZONE)
          )
          ->format(DateTimeItemInterface::DATETIME_STORAGE_FORMAT);
        /** @var \Drupal\file\Plugin\Field\FieldType\FileItem $mandate_field_item */
        $mandate_field_item = $commerce_order->sepa_mandate->isEmpty() ?
          $commerce_order->sepa_mandate->appendItem() :
          $commerce_order->sepa_mandate->first();
        $mandate_file = $this->client->getBinaryContent(
          $slimpay_mandate,
          $mandate_field_item->getUploadLocation()
        );
        if ($mandate_file instanceof FileInterface) {
          $commerce_order->sepa_mandate->target_id = $mandate_file->id();
        }
      }
      if ($slimpay_document = $this->client->getDocument($slimpay_order)) {
        /** @var \Drupal\file\Plugin\Field\FieldType\FileItem $contract_field_item */
        $contract_field_item = $commerce_order->contract->isEmpty() ?
          $commerce_order->contract->appendItem() :
          $commerce_order->contract->first();
        $contract_file = $this->client->getBinaryContent(
          $slimpay_document,
          $contract_field_item->getUploadLocation()
        );
        if ($contract_file instanceof FileInterface) {
          $commerce_order->contract->target_id = $contract_file->id();
        }
      }
      $commerce_order->save();
    }
    else {
      new BadRequestHttpException(
        "Invalid order state: {$slimpay_order_state['state']}"
      );
    }

    $payment->setRemoteState($slimpay_order_state['state']);
    $payment->save();
  }

  /**
   * Gets the warning form render array.
   *
   * @return array
   *   The render array.
   */
  public function getWarning() {
    return [
      '#type' => 'processed_text',
      '#text' => $this->configuration['warning'],
      '#format' => 'plain_text',
    ];
  }

}
