<?php

namespace Drupal\slimpay_api\PluginForm;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\slimpay_api\Client;
use HapiClient\Exception\HttpException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Iframe integration for direct debit payments.
 */
class DirectDebitForm extends PaymentOffsiteForm implements ContainerInjectionInterface {

  /**
   * The Slimpay API client.
   *
   * @var \Drupal\slimpay_api\Client
   */
  protected $client;

  /**
   * HypermediaApi constructor.
   *
   * @param \Drupal\slimpay_api\Client $client
   *   The Slimpay API client.
   */
  public function __construct(Client $client) {
    $this->client = $client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('slimpay_api.client')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state
  ) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    /** @var \Drupal\commerce_mollie\Plugin\Commerce\PaymentGateway\DirectDebit $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $payment->getOrder();
    /** @var \Drupal\profile\Entity\ProfileInterface $profile */
    $billing_profile = $order->getBillingProfile();
    /** @var \Drupal\address\AddressInterface $address */
    $address = $billing_profile->address->first();

    $time = time();
    $order_remote_id = "{$order->id()}-{$time}";
    $subscriber_remote_id = "{$billing_profile->id()}-{$time}";

    $success_url = Url::fromRoute(
      'slimpay_api.checkout.slimpay_return',
      ['commerce_order' => $order->id()],
      ['absolute' => TRUE]
    )->toString();

    // Create Slimpay's order.
    try {
      $order = $this->client->createOrders(
        $order_remote_id,
        $subscriber_remote_id,
        $address->getAddressLine1(),
        $address->getAddressLine2(),
        $address->getLocality(),
        $address->getPostalCode(),
        $address->getCountryCode(),
        $billing_profile->civility->value,
        $address->getFamilyName(),
        $address->getGivenName(),
        $billing_profile->email->value,
        $billing_profile->mobile_phone->value,
        $billing_profile->iban->value,
        $billing_profile->customer_number->value,
        $success_url,
        $payment_gateway_plugin->getCancelUrl(),
        $payment_gateway_plugin->getFailureUrl()
      );
    }
    catch (HttpException $e) {
      throw new PaymentGatewayException(
        'Failed to create order.',
        0,
        NULL,
        $e->getResponseBody()
      );
    }
    catch (\Exception $e) {
      throw new PaymentGatewayException(
        'Failed to create order.',
        0,
        NULL,
        $e->getMessage()
      );
    }

    $payment->setRemoteId($order_remote_id);
    $payment->setRemoteState($order->getState()['state']);
    $payment->save();

    // Get Slimpay's extended aproval.
    try {
      $extended_approval = $this->client->extendedUserApproval($order);
    }
    catch (HttpException $e) {
      throw new PaymentGatewayException(
        'Failed to get extended approval.',
        0,
        NULL,
        $e->getResponseBody()
      );
    }
    catch (\Exception $e) {
      throw new PaymentGatewayException(
        'Failed to get extended approval.',
        0,
        NULL,
        $e->getMessage()
      );
    }

    $iframe_code = base64_decode($extended_approval->getState()['content']);
    $progress_bar = '<div class="progress"><div class="progress-bar progress-bar-striped progress-bar-animated" role="progressbar" aria-valuenow="75" aria-valuemin="0" aria-valuemax="100" style="width: 75%"></div></div>';
    $form['warning'] = $payment_gateway_plugin->getWarning();
    $form['extended-user-approval'] = [
      '#type' => 'inline_template',
      '#template' => '<div>' . $progress_bar . $iframe_code . '</div>',
      '#attached' => [
        'library' => ['slimpay_api/progress'],
      ],
    ];

    return $form;
  }

}
